# Generated by Django 4.2.5 on 2023-09-19 08:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('ievv_api_key', '0002_auto_20201119_1956'),
    ]

    operations = [
        migrations.CreateModel(
            name='ScopedApiKeyIssuedToken',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('jti', models.CharField(editable=False, max_length=255, unique=True)),
                ('token', models.TextField(editable=False)),
                ('token_payload', models.JSONField(editable=False)),
                ('issued_at', models.DateTimeField(editable=False)),
                ('expires_at', models.DateTimeField(editable=False)),
                ('scoped_api_key', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ievv_api_key.scopedapikey')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ScopedApiKeyBlacklistedToken',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('blacklisted_at', models.DateTimeField(auto_now_add=True)),
                ('token', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ievv_jwt_blacklist_api_key.scopedapikeyissuedtoken')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
